/*utf8解析行 Don't remove or modify this first line!*/
#ifndef WNBITINDEX_H
#define WNBITINDEX_H

#include "WNMemoryObjectManger.h"

class WNBitNode
{
public:
	WNBitNode();
	virtual ~WNBitNode();
	//初始化
	void ReInit();
	//得到指定节点值(0-15)的子节点
	WNBitNode* ChildNode(unsigned int uChildValue);
	//得到指定节点值(0-15)的子节点的引用
	WNBitNode*& ChildNodeRef(unsigned int uChildValue);
	//添加子节点pNewNode
	//如果pNewNode对应位置(pNewNode->muBitValue)的子节点已存在且bRemoveOldNode为false，则返回已有节点
	//如果bRemoveOldNode为true则delete已有节点，同时返回NULL
	//如果pNewNode与已存在的子节点相同则返回NULL
	WNBitNode* AttatchChildNode(WNBitNode* pNewNode, bool bRemoveOldNode = true);
	//分离指定节点值(0-15)的子节点
	//返回已存在的节点，如指定节点不存在则返回NULL
	WNBitNode* DetatchChildNode(unsigned int uChildValue);
	//增加指定节点值(0-15)的子节点
	//返回新增的节点
	//如果节点已存在，则返回已有节点
	WNBitNode* AddChildNode(unsigned int uChildValue);
	//移除指定节点值(0-15)的子节点
	void RemoveChildNode(unsigned int uChildValue);
	//得到当前的节点的子节点数目
	unsigned int LeafsCount();
	//得到当前节点的子节点数目(包含数据)
	unsigned int LeafsAndValueCount();
	//获得当前节点所对应的数据
	void* Data();
	//获得当前节点所对应的数据的引用
	void*& DataRef();
	//设置当前节点所对应的数据为pNewData
	//如果pNewData为NULL则删除当前节点所对应的数据的指针,此时如果mbIsWNObjectPtr为true则会旧数据进行Release操作
	//如果bIsWNObject为true则会对新数据进行Retain操作
	//返回旧数据，当旧数据(当前mbIsWNObjectPtr)为true时返回NULL
	void* SetData(void* pNewData, bool bIsWNObject = false);
public:
	static WNBitNode* NullPtr; //用于返回空引用
public:
	unsigned long muLeafsInfo : 16; //叶子节点列表信息
	unsigned int muNodeValue : 12; //当前节点所代表的值
	bool mbHasData  : 1; //当前节点是否有数值
	bool mbIsWNObjectPtr : 1; //表示当前节点的数据是否是WNObject*类型的数据
							  //如果为true则析构或者调用SetData时会进行相关引用操作
	bool mbIsEndBit : 1; //是否是一个字节的结束
	bool mbDoNotFreeLeafs : 1; //是否在析构时delete所有的子节点，为true表示不delete
	WNBitNode** mppLeafs; //叶子节点以及数据指针
};



class WNBitIndex
{
public:
	typedef WNMemoryObjectManager<WNBitNode> NodeHeapManager;
public:
	WNBitIndex();
	virtual ~WNBitIndex();
	//设置指定关键字的映射
	//pKey是要设置的关键字，不能为NULL
	//uKeyLen是关键字的长度，不能为0，单位是字节
	//pData是与pKey关联的数据
	//如果关键字已存在，则使用pData替换已存在的数据
	//如果pData为NULL表示删除与pKey相关联的数据
	//当pData以及pKey均不为NULL时返回与pKey相关联的旧数据(旧数据不存在返回NULL)
	//如果旧数据是WNObject类型，返回NULL(参见WNBitNode类的SetData函数)
	virtual void* SetItem(const void* pKey, unsigned int uKeyLen, void* pData, bool mbIsWNObject = false);
	//获得与指定关键字相关联的数据
	//pKey是指定的关键字，不能为NULL
	//uKeyLen是关键字长度，不能为0，单位是字节
	//返回值是与pKey相关联的数据，为NULL则表示数据不存在
	virtual void* GetItem(const void* pKey, unsigned int uKeyLen);
	//移除与指定关键字相关联的数据
	//pKey是指定的关键字，不能为NULL
	//uKeyLen是关键字长度，单位是字节
	//返回值是与pKey相关联的数据，为NULL则表示数据不存在
	virtual void* RemoveItem(const void* pKey, unsigned int uKeyLen);
	//清空所有节点
	virtual void ClearAll();
protected:
	//查找与指定关键字相同的节点
	//pKey是指定的关键字，不能为NULL
	//uKeyLen是关键字长度，单位是字节
	//返回值是与pKey相关联的节点
	virtual WNBitNode* FindNode(const void* pKey, unsigned int uKeyLen);
	//添加与指定关键字相同的节点(指定关键字的最后半个字节的节点)
	//pKey是指定的关键字，不能为NULL
	//uKeyLen是关键字长度,单位是字节
	//返回值是与pKey相关联的节点，如果关键字节点不存在则添加
	virtual WNBitNode* AddNode(const void* pKey, unsigned int uKeyLen);
protected:
	WNBitNode* mpRoot[256]; //所有根节点，数组，下标从0-255
	NodeHeapManager* mpNodeHeapManager; //所有中间节点的内存管理
};

#endif
