/*utf8解析行 Don't remove or modify this first line!*/
#ifndef VNETMATELIST_H

#define VNETMATELIST_H


#include <unistd.h>




template <class T>

class WNCLoopListNode

{

public:

	WNCLoopListNode() { prevNode = 0; nextNode = 0; };

	WNCLoopListNode(const T& tValue) { prevNode = 0; nextNode = 0; data = tValue; }


	T data;

	WNCLoopListNode* prevNode;

	WNCLoopListNode* nextNode;

};



template <class T>

class WNCLoopList

{

public:

	typedef WNCLoopListNode<T> Node;

	typedef Node* PNode;




	WNCLoopList()

	{

		m_head = new Node;

		m_head->prevNode = m_head;

		m_head->nextNode = m_head;

		m_begin = m_head;

		m_end = m_head;

		m_current = m_head;

		m_dwCount = 0;
	

	}

	WNCLoopList(const WNCLoopList<T>& list)
	{
		m_head = new Node;

		m_head->prevNode = m_head;

		m_head->nextNode = m_head;

		m_begin = m_head;

		m_end = m_head;

		m_current = m_head;

		m_dwCount = 0;

		*this = list;
	}

	WNCLoopList<T>& operator = (const WNCLoopList<T>& list)
	{
		ClearList();
		for (WNCLoopList<T>::PNode pNode = list.Begin(); pNode != list.Head(); pNode = pNode->nextNode)
		{
			AppendNode(pNode->data);
		}
		return *this;
	}

	virtual ~WNCLoopList()

	{

		ClearList();

		if (m_head)

		{

			delete m_head;

			m_head = 0;

		}
	
		m_begin = 0;

		m_end = 0;

		m_current = 0;

		m_dwCount = 0;

	}

	inline PNode Head() const { return m_head; }



	inline PNode Begin() const { return m_begin; }



	inline PNode End() const { return m_end; }



	inline PNode Current() const { return m_current; }



	inline PNode MoveBegin() { return m_current = m_begin; }



	inline PNode MoveEnd() { return m_current = m_end; }



	inline PNode MovePrev() { return m_current = m_current->prevNode; }



	inline PNode MoveNext() { return m_current = m_current->nextNode; }



	inline unsigned long ListCount() const { return m_dwCount; }



	inline T& HeadData() const { return m_head->data; }



	inline T& BeginData() const { return m_begin->data; }



	inline T& EndData() const { return m_end->data; }



	inline T& CurrentData() const { return m_current->data; }



	inline bool NotHead() const { return m_current != m_head; }



	PNode Item(unsigned long dwIndex) const

	{

		PNode pNode = m_head;

		while (dwIndex)

		{

			pNode = pNode->nextNode;

			dwIndex--;

		}

		return pNode;

	}



	inline T& operator [] (unsigned long dwIndex) const

	{

		return Item(dwIndex)->data;

	}

	bool FindNode(PNode pFindNode) const
	{
		for (PNode pNode = m_begin; pNode != m_head; pNode = pNode.nextNode)
		{
			if (pNode == pFindNode)
			{
				return true;
			}
		}
		return false;
	}

	PNode FindData(T& data) const
	{
		for (PNode pNode = m_begin; pNode != m_head; pNode = pNode->nextNode)
		{
			if (pNode->data == data)
			{
				return pNode;
			}
		}
		return 0;
	}

	PNode AppendNode(const Node& node, PNode pPrevNode = 0)

	{

		PNode pNode = (PNode)&node;

		if (!pNode)

		{

			return 0;

		}

		if (!pPrevNode)

		{

			pPrevNode = m_end;

		}



		PNode pNewNode = new Node(*pNode);



		pNewNode->nextNode = pPrevNode->nextNode;

		pPrevNode->nextNode->prevNode = pNewNode;



		pPrevNode->nextNode = pNewNode;

		pNewNode->prevNode = pPrevNode;



		if (pNewNode->prevNode == m_head)

		{

			m_begin = pNewNode;

		}

		if (pNewNode->nextNode == m_head)

		{

			m_end = pNewNode;

		}

		m_dwCount++;

		return pNewNode;

	}



	PNode AppendNode(PNode pNode, PNode pPrevNode = 0)

	{
		if (!pNode)

		{

			return 0;

		}

		if (!pPrevNode)

		{

			pPrevNode = m_end;

		}



		PNode pNewNode = pNode;

	
	
		
		pNewNode->nextNode = pPrevNode->nextNode;

		pPrevNode->nextNode->prevNode = pNewNode;
		


		pPrevNode->nextNode = pNewNode;

		pNewNode->prevNode = pPrevNode;
	


		if (pNewNode->prevNode == m_head)

		{

			m_begin = pNewNode;

		}

		if (pNewNode->nextNode == m_head)

		{

			m_end = pNewNode;

		}

		m_dwCount++;

		return pNewNode;

	}



	PNode AppendNode(T& data, PNode pPreNode = 0)

	{

		PNode pNode = new Node;

		pNode->data = data;

		return AppendNode(pNode, pPreNode);

	}



	PNode PreAppendNode(const Node& node, PNode pNextNode = 0)

	{

		PNode pNode = (PNode)&node;

		if (!pNode)

		{

			return NULL;

		}

		if (!pNextNode)

		{

			pNextNode = m_begin;

		}



		PNode pNewNode = new Node(*pNode);





		pNewNode->prevNode = pNextNode->prevNode;

		pNextNode->prevNode->nextNode = pNewNode;



		pNewNode->nextNode = pNextNode;

		pNextNode->prevNode = pNewNode;



		if (pNewNode->prevNode == m_head)

		{

			m_begin = pNewNode;

		}

		if (pNewNode->nextNode == m_head)

		{

			m_end = pNewNode;

		}

		m_dwCount++;

		return pNewNode;

	}



	PNode PreAppendNode(PNode pNode, PNode pNextNode = 0)

	{

		if (!pNode)

		{

			return NULL;

		}

		if (!pNextNode)

		{

			pNextNode = m_begin;

		}



		PNode pNewNode = pNode;





		pNewNode->prevNode = pNextNode->prevNode;

		pNextNode->prevNode->nextNode = pNewNode;



		pNewNode->nextNode = pNextNode;

		pNextNode->prevNode = pNewNode;



		if (pNewNode->prevNode == m_head)

		{

			m_begin = pNewNode;

		}

		if (pNewNode->nextNode == m_head)

		{

			m_end = pNewNode;

		}

		m_dwCount++;

		return pNewNode;

	}



	PNode PreAppendNode(T& data, PNode pNextNode = 0)

	{

		PNode pNode = new Node;

		pNode->data = data;

		return PreAppendNode(pNode, pNextNode);

	}

	PNode DetachNode(PNode pNode = 0)
	{
		if (!pNode)
		{
			pNode = m_current;
		}

		if (pNode == m_head)
		{
			return 0;
		}


		if (m_current == pNode)
		{
			if (m_current == m_end)
			{
				m_current = pNode->prevNode;
			}
			else
			{
				m_current = pNode->nextNode;
			}
		}

		pNode->prevNode->nextNode = pNode->nextNode;
		if (m_end == pNode)
		{
			m_end = pNode->prevNode;
		}
		pNode->nextNode->prevNode = pNode->prevNode;
		if (m_begin == pNode)
		{
			m_begin = pNode->nextNode;
		}
		pNode->prevNode= NULL;
		pNode->nextNode = NULL;
		m_dwCount--;
		return pNode;
	}

	PNode RemoveNode(PNode pNode = 0)

	{

		if (!pNode)

		{

			pNode = m_current;

		}

		if (pNode == m_head)

		{

			return m_head;

		}

		PNode retNode = 0;



		if (m_current == pNode)

		{

			if (m_current == m_end)

			{

				m_current = pNode->prevNode;

			}

			else

			{

				m_current = pNode->nextNode;

			}

			retNode = m_current;

		}

		else

		{

			if (pNode == m_end)

			{

				retNode = pNode->prevNode;

			}

			else

			{

				retNode = pNode->nextNode;

			}

		}





		pNode->prevNode->nextNode = pNode->nextNode;

		if (m_end == pNode)

		{

			m_end = pNode->prevNode;

		}

		pNode->nextNode->prevNode = pNode->prevNode;

		if (m_begin == pNode)

		{

			m_begin = pNode->nextNode;

		}

		delete pNode;

		m_dwCount--;

		return retNode;

	}



	virtual void ClearList()

	{

		for (MoveBegin(); Current() != Head(); )

		{

			RemoveNode();

		}

		m_begin = m_head;

		m_end = m_head;

		m_current = m_head;

		m_dwCount = 0;

	}



protected:

	PNode m_head;

	PNode m_begin;

	PNode m_end;

	PNode m_current;

	unsigned long m_dwCount;

};







#endif
