/*utf8解析行 Don't remove or modify this first line!*/
#ifndef MEMORYOBJECTMANAGER_H
#define MEMORYOBJECTMANAGER_H

#include "WNCLoopList.h"

template <class T>
class WNMemoryObject
{
public:
    WNMemoryObject() { m_pPrevObject = NULL; m_pNextObject = NULL; }
public:
    T m_Object;
    WNMemoryObject* m_pPrevObject;
    WNMemoryObject* m_pNextObject;
};

template <class T>
class WNMemoryObjectManager
{
public:
    typedef WNMemoryObject<T> Node;
    typedef Node* PNode;
    typedef WNCLoopList<PNode> MemoryBlocks;
public:
    WNMemoryObjectManager(unsigned long dwBlockSize = 1024)
    {
        m_dwBlockSize = dwBlockSize;
        m_FreeObjectHead.m_pNextObject = &m_FreeObjectHead;
        m_FreeObjectHead.m_pPrevObject = &m_FreeObjectHead;
        m_dwFreeCount = 0;
        m_UsedObjectHead.m_pNextObject = &m_UsedObjectHead;
        m_UsedObjectHead.m_pPrevObject = &m_UsedObjectHead;
        m_dwUsedCount = 0;
        AllocMemoryBlock();
    }

    virtual ~WNMemoryObjectManager()
    {
        for (m_MemoryBlocks.MoveBegin(); m_MemoryBlocks.NotHead(); m_MemoryBlocks.MoveNext())
        {
            delete[] m_MemoryBlocks.CurrentData();
        }
    }

    //从自由区分配对象指针，返回的对象指针未经过初始化
    virtual T* AllocObject()
    {
        if (!m_dwFreeCount)
        {
            AllocMemoryBlock();
        }
        if (!m_dwFreeCount)
        {
            return NULL;
        }
        PNode pNode = m_FreeObjectHead.m_pNextObject;
        //将节点从自由区移入已使用区
        m_FreeObjectHead.m_pNextObject = pNode->m_pNextObject;
        pNode->m_pNextObject->m_pPrevObject = &m_FreeObjectHead;
        m_dwFreeCount--;
        m_UsedObjectHead.m_pPrevObject->m_pNextObject = pNode;
        pNode->m_pPrevObject = m_UsedObjectHead.m_pPrevObject;
        m_UsedObjectHead.m_pPrevObject = pNode;
        pNode->m_pNextObject = &m_UsedObjectHead;
        m_dwUsedCount--;
        return (T*)pNode;
    }

    //将对象指针从使用区放入自由区，pObject必须为从AllocObject函数分配的对象指针
    virtual void FreeObject(T* pObject)
    {
        PNode pNode = (PNode)pObject;
        //将节点从使用区移入自由区
        pNode->m_pPrevObject->m_pNextObject = pNode->m_pNextObject;
        pNode->m_pNextObject->m_pPrevObject = pNode->m_pPrevObject;
        m_dwUsedCount--;
        m_FreeObjectHead.m_pPrevObject->m_pNextObject = pNode;
        pNode->m_pPrevObject = m_FreeObjectHead.m_pPrevObject;
        m_FreeObjectHead.m_pPrevObject = pNode;
        pNode->m_pNextObject = &m_FreeObjectHead;
        m_dwFreeCount++;
    }
protected:
    virtual bool AllocMemoryBlock()
    {
        if (!m_dwBlockSize)
        {
            return false;
        }
        PNode pBlock = new Node[m_dwBlockSize];
        if (!pBlock)
        {
            return false;
        }
        //将内存块放入内存块列表
        m_MemoryBlocks.AppendNode(pBlock);
        //将所有区块相连
        for (unsigned long i = 0; i < m_dwBlockSize; i++ )
        {
            pBlock[i].m_pPrevObject = pBlock + i - 1;
            pBlock[i].m_pNextObject = pBlock + i + 1;
        }
        //将区块加入自由区
        m_FreeObjectHead.m_pPrevObject->m_pNextObject = pBlock;
        pBlock->m_pPrevObject = m_FreeObjectHead.m_pPrevObject->m_pNextObject;
        m_FreeObjectHead.m_pPrevObject = &pBlock[m_dwBlockSize - 1];
        pBlock[m_dwBlockSize -1 ].m_pNextObject = &m_FreeObjectHead;
        m_dwFreeCount += m_dwBlockSize;
        return true;
    }
protected:
    MemoryBlocks m_MemoryBlocks;
    Node m_FreeObjectHead;
    Node m_UsedObjectHead;
    unsigned long m_dwBlockSize;
    unsigned long m_dwFreeCount;
    unsigned long m_dwUsedCount;
};

#endif