/*utf8解析行 Don't remove or modify this first line!*/
#include "WNSocketSelectObserver.h"
#include "WNSocketObserverManager.h"
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>

WNSocketSelectObserver mWNSocketSelectObserver;

WNSocketSelectObserver::WNSocketSelectObserver()
    :mdwThreadId(-1)
	,mThreadHandle(0)
	,mbIsWorking(false)
	,mbExitThread(false)
	,mpOperateLocker(0)
{
	CreateLocker();
	signal(SIGPIPE,SIG_IGN);
	WNSocketObserverManager::RegisterObserver(this);
    //if (this == &mWNSocketSelectObserver)
    //{
    //    printf("enter mWNSocketSelectObserver WNSocketSelectObserver::WNSocketSelectObserver\n");
    //}
    //else
    //{
    //    printf("enter WNSocketSelectObserver::WNSocketSelectObserver\n");
    //}
}

WNSocketSelectObserver::~WNSocketSelectObserver()
{
    //if (this == &mWNSocketSelectObserver)
    //{
    //    printf("enter mWNSocketSelectObserver WNSocketSelectObserver::~WNSocketSelectObserver\n");
    //}
    //else
    //{
    //    printf("enter WNSocketSelectObserver::~WNSocketSelectObserver\n");
    //}
	StopServer();
	DestroyLocker();
    //if (this == &mWNSocketSelectObserver)
    //{
    //    printf("exit mWNSocketSelectObserver WNSocketSelectObserver::~WNSocketSelectObserver\n");
    //}
    //else
    //{
    //    printf("exit WNSocketSelectObserver::~WNSocketSelectObserver\n");
    //}
}


bool WNSocketSelectObserver::AddSocket(WNSocketObserverTarget* socketInfo)
{
	Lock();
	bool bRet = InternalAddSocket(socketInfo);
	Unlock();
	return bRet;
}


bool WNSocketSelectObserver::RemoveSocket( WNSocketObserverTarget* socketInfo )
{
	Lock();
	bool bRet = InternalRemoveSocket(socketInfo);
	Unlock();
	return bRet;
}


bool WNSocketSelectObserver::StartServer()
{
	Lock();
	bool bRet = InternalStartServer();
	Unlock();
	return bRet;
}


bool WNSocketSelectObserver::StopServer()
{
	Lock();
	bool bRet = InternalStopServer();
	Unlock();
	return bRet;
}


const char* WNSocketSelectObserver::GetServerName()
{
	return "WNSocketSelectObserver";
}


void* WNSocketSelectObserver::SelectThreadProc(void* lpParameter)
{
	WNSocketSelectObserver* pObserver = (WNSocketSelectObserver*)lpParameter;
	if (pObserver)
	{
		pObserver->InternalSelectThreadProc();
	}
	else
	{
		return 0;
	}
	return 0;
}

long WNSocketSelectObserver::InternalSelectThreadProc()
{
	//设置当前状态为正在工作
	mbIsWorking = true;
	
	WNSocketObserverTargetList::PNode pSocketNode = NULL;
	fd_set sockReadSet;
	FD_ZERO(&sockReadSet);
    timeval timeOut = { 0, 0 };
	int iSelectRet = 0;
	int iSelectLoopCount = 0;
	int iSelectEventCount = 0;
	WNSocketObserverTargetDelegate* pTargetDelegate = NULL;

	while (!mbExitThread)
	{
		//sleep(5);
		//检查socket状态
		if (!TryLock())
		{
			continue;
		}
		iSelectEventCount = 0;
		for (pSocketNode = mTargetList.Begin(); pSocketNode != mTargetList.Head(); pSocketNode = pSocketNode->nextNode)
		{
			if (!pSocketNode->data->mpObserverTargetDelegate)
			{
				//没有托管，不进行读检查
				//Sleep(5);
				continue;
			}
			//if (pSocketNode->data->TryLock())
			{
				//尝试锁定socket操作成功
				//首先设置读socket集合
				FD_ZERO(&sockReadSet);
				FD_SET(pSocketNode->data->mSocket, &sockReadSet);
				iSelectRet = select(pSocketNode->data->mSocket + 1, &sockReadSet, NULL, NULL, &timeOut);
				if (!iSelectRet)
				{
					//没有socket状态改变
					//struct sockaddr_in  addr;
					//int iSize = sizeof(addr);
					//getsockname(pSocketNode->data->mSocket, (struct sockaddr*)&addr, (socklen_t*)&iSize);
					//char* paddr = inet_ntoa(((struct sockaddr_in*)&addr)->sin_addr);
					//unsigned int uPort = ntohs(((struct sockaddr_in *)&addr)->sin_port);
					goto CONTINUE_NEXT_SELECT;
				}
				else if (iSelectRet != -1)
				{
					
					
					//有读的内容
					//首先停止下一次的select
					pTargetDelegate = pSocketNode->data->mpObserverTargetDelegate;
					pSocketNode->data->mpObserverTargetDelegate = NULL;
					pTargetDelegate->OnObserverRead(this);
					iSelectEventCount++;
				}
				else
				{
					
					//select错误
					//首先停止下一次的select
					pTargetDelegate = pSocketNode->data->mpObserverTargetDelegate;
					pSocketNode->data->mpObserverTargetDelegate = NULL;
                    pSocketNode->data->miObserverErrorCode = -1;//(int)errno;//::WSAGetLastError();
					pTargetDelegate->OnObserverError(this);
					iSelectEventCount++;
				}
CONTINUE_NEXT_SELECT:
                ;
				//pSocketNode->data->Unlock();
			}
		}
		Unlock();
		if (!iSelectEventCount)
		{
			
                //没有事件发生，记录没有事件发生的轮询次数
                iSelectLoopCount++;
                if (iSelectLoopCount == 1)
                {
                    //10次轮询都没有事件发生，检测消息队列
//                    while(::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
//                    {
//                        ::TranslateMessage(&msg);
//                        ::DispatchMessage(&msg);
//                    }
                    //线程进入睡眠状态
                    ::usleep(5000);
                    //没有事件发生的轮询次数归零
                    iSelectLoopCount = 0;
                }
		}
		else
		{
			//有事件发生，没有事件发生的轮询次数归零
			iSelectLoopCount = 0;
		}

        //sleep(5);
	}
	//设置当前状态为没有工作
	mbIsWorking = false;
	return 0;
}

bool WNSocketSelectObserver::InternalStartServer()
{
	//if (!mdwThreadId)
	{
		//mThreadHandle = ::CreateThread(NULL, 0, &WNSocketSelectObserver::SelectThreadProc, this, 0, &mdwThreadId);
		pthread_create(&mdwThreadId, NULL, &WNSocketSelectObserver::SelectThreadProc, (void *)this);
		pthread_detach(mdwThreadId);
	}
    while (!IsWorking())
    {
        ::usleep(500);
    }
	return true;
}

bool WNSocketSelectObserver::InternalStopServer()
{
    //printf("enter WNSocketSelectObserver::InternalStopServer()\n");
	if (-1 != mdwThreadId)
	{
        //printf("-1 != mdwThreadId(%d)\n", (int)pthread_self());
		mbExitThread = true;
		unsigned long dwExitCode;
		unsigned long dwThreadExitTime = 0;
		
        //printf("while (IsWorking())\n");
        while (IsWorking())
        {
            ::usleep(5 * 1000);
        }
        mbExitThread = false;
        mdwThreadId = -1;
	}
    //printf("exit WNSocketSelectObserver::InternalStopServer()\n");
	return true;
}

bool WNSocketSelectObserver::InternalAddSocket( WNSocketObserverTarget* socketInfo )
{
	if (mTargetList.FindData(socketInfo))
	{
		return true;
	}
	if (!mTargetList.AppendNode(socketInfo))
	{
		return false;
	}
	if (!mbIsWorking)
	{
		//开启服务
		InternalStartServer();
	}
	return true;
}

bool WNSocketSelectObserver::InternalRemoveSocket( WNSocketObserverTarget* socketInfo )
{
	WNSocketObserverTargetList::PNode pNode = NULL;
	pNode = mTargetList.FindData(socketInfo);
	if (!pNode)
	{
		return false;
	}
	mTargetList.RemoveNode(pNode);
	if (!mTargetList.ListCount())
	{
		//没有需要监控的socket
		//停止服务
		InternalStopServer();
	}
	return true;
}


void WNSocketSelectObserver::CreateLocker()
{
	mpOperateLocker = new WNCreticalSection;
	
}

void WNSocketSelectObserver::DestroyLocker()
{
	if (mpOperateLocker)
	{
		delete mpOperateLocker;
		mpOperateLocker = NULL;
	}
}

bool WNSocketSelectObserver::TryLock()
{
	if (mpOperateLocker)
	{
		return mpOperateLocker->TryLock();
	}
	else
	{
		return false;
	}
}

bool WNSocketSelectObserver::Lock()
{
	if (mpOperateLocker)
	{
		mpOperateLocker->Lock();
	}
	return false;
}

bool WNSocketSelectObserver::Unlock()
{
	if (mpOperateLocker)
	{
		mpOperateLocker->Unlock();
	}
	return false;
}

bool WNSocketSelectObserver::IsWorking()
{
	return mbIsWorking;
}


